"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const realFs = require("fs");
const getStream = require("get-stream");
const InlineTransformer_1 = require("./InlineTransformer");
const iterateSources_1 = require("./iterateSources");
const ProcessSnapshot_1 = require("./ProcessSnapshot");
const TransformRunner_1 = require("./TransformRunner");
class RunResult {
    constructor(stats) {
        this.stats = stats;
    }
}
exports.RunResult = RunResult;
class RunStats {
    constructor(modified = 0, errors = 0, total = 0) {
        this.modified = modified;
        this.errors = errors;
        this.total = total;
    }
}
exports.RunStats = RunStats;
class CLIEngine {
    constructor(config, onTransform = () => { }, readStdin = () => __awaiter(this, void 0, void 0, function* () { return yield getStream(process.stdin); }), writeStdout = (data) => process.stdout.write(data), fs = realFs) {
        this.config = config;
        this.onTransform = onTransform;
        this.readStdin = readStdin;
        this.writeStdout = writeStdout;
        this.fs = fs;
    }
    loadPlugins() {
        return __awaiter(this, void 0, void 0, function* () {
            let snapshot = new ProcessSnapshot_1.default();
            let plugins;
            try {
                this.config.loadBabelTranspile();
                this.config.loadRequires();
                plugins = yield this.config.getBabelPlugins();
            }
            finally {
                this.config.unloadBabelTranspile();
                snapshot.restore();
            }
            return plugins;
        });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            var e_1, _a;
            let plugins = yield this.loadPlugins();
            let runner;
            let modified = 0;
            let errors = 0;
            let total = 0;
            let dryRun = this.config.dry;
            let sourcesIterator;
            if (this.config.stdio) {
                sourcesIterator = [new TransformRunner_1.Source('<stdin>', yield this.readStdin())][Symbol.iterator]();
            }
            else {
                sourcesIterator = iterateSources_1.default(this.config.sourcePaths, this.config.extensions, this.config.ignore, this.fs.statSync, this.fs.readdirSync, this.fs.readFileSync);
            }
            runner = new TransformRunner_1.default(sourcesIterator, new InlineTransformer_1.default(plugins));
            try {
                for (var _b = __asyncValues(runner.run()), _c; _c = yield _b.next(), !_c.done;) {
                    let result = yield _c.value;
                    this.onTransform(result);
                    if (result.kind === TransformRunner_1.SourceTransformResultKind.Transformed) {
                        if (this.config.stdio) {
                            this.writeStdout(result.output);
                        }
                        else {
                            if (result.output !== result.source.content) {
                                modified++;
                                if (!dryRun) {
                                    this.fs.writeFileSync(result.source.path, result.output);
                                }
                            }
                        }
                    }
                    else if (result.error) {
                        errors++;
                    }
                    total++;
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            return new RunResult(new RunStats(modified, errors, total));
        });
    }
}
exports.default = CLIEngine;
//# sourceMappingURL=CLIEngine.js.map