import { BabelPlugin, RawBabelPlugin } from './BabelPluginTypes';
import { PathPredicate } from './iterateSources';
export declare class Plugin {
    readonly rawPlugin: RawBabelPlugin;
    readonly inferredName: string;
    readonly source?: string | undefined;
    readonly resolvedPath?: string | undefined;
    readonly declaredName?: string;
    constructor(rawPlugin: RawBabelPlugin, inferredName: string, source?: string | undefined, resolvedPath?: string | undefined);
}
export declare enum Printer {
    Recast = "recast",
    Prettier = "prettier",
    Babel = "babel"
}
export default class Config {
    readonly sourcePaths: Array<string>;
    readonly localPlugins: Array<string>;
    readonly remotePlugins: Array<string>;
    readonly pluginOptions: Map<string, object>;
    readonly printer: Printer;
    readonly extensions: Set<string>;
    readonly requires: Array<string>;
    readonly transpilePlugins: boolean;
    readonly findBabelConfig: boolean;
    readonly ignore: PathPredicate;
    readonly stdio: boolean;
    readonly dry: boolean;
    constructor(sourcePaths?: Array<string>, localPlugins?: Array<string>, remotePlugins?: Array<string>, pluginOptions?: Map<string, object>, printer?: Printer, extensions?: Set<string>, requires?: Array<string>, transpilePlugins?: boolean, findBabelConfig?: boolean, ignore?: PathPredicate, stdio?: boolean, dry?: boolean);
    private pluginLoader;
    private remotePluginLoader;
    private _pluginCache?;
    getPlugins(): Promise<Array<Plugin>>;
    loadRequires(): void;
    loadBabelTranspile(): void;
    unloadBabelTranspile(): void;
    getPlugin(name: string): Promise<Plugin | null>;
    getBabelPlugins(): Promise<Array<BabelPlugin>>;
    getBabelPlugin(name: string): Promise<BabelPlugin | null>;
}
export declare class ConfigBuilder {
    private _sourcePaths?;
    private _localPlugins?;
    private _remotePlugins?;
    private _pluginOptions?;
    private _printer?;
    private _extensions;
    private _requires?;
    private _transpilePlugins?;
    private _findBabelConfig?;
    private _ignore?;
    private _stdio?;
    private _dry?;
    sourcePaths(value: Array<string>): this;
    addSourcePath(value: string): this;
    addSourcePaths(...values: Array<string>): this;
    localPlugins(value: Array<string>): this;
    addLocalPlugin(value: string): this;
    remotePlugins(value: Array<string>): this;
    addRemotePlugin(value: string): this;
    pluginOptions(value: Map<string, object>): this;
    setOptionsForPlugin(options: object, plugin: string): this;
    printer(value: Printer): this;
    extensions(value: Set<string>): this;
    addExtension(value: string): this;
    requires(value: Array<string>): this;
    addRequire(value: string): this;
    transpilePlugins(value: boolean): this;
    findBabelConfig(value: boolean): this;
    ignore(value: PathPredicate): this;
    stdio(value: boolean): this;
    dry(value: boolean): this;
    build(): Config;
}
