"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@babel/core");
const path_1 = require("path");
const pirates_1 = require("pirates");
const AllSyntaxPlugin_1 = require("./AllSyntaxPlugin");
const extensions_1 = require("./extensions");
let useBabelrc = false;
let revert = null;
function hook(code, filename) {
    let ext = path_1.extname(filename);
    if (!extensions_1.PluginExtensions.has(ext)) {
        throw new Error(`cannot load file type '${ext}': ${filename}`);
    }
    let options = {
        filename,
        babelrc: useBabelrc,
        presets: [],
        plugins: [AllSyntaxPlugin_1.default],
        sourceMaps: 'inline'
    };
    if (!useBabelrc) {
        if (extensions_1.TypeScriptExtensions.has(ext)) {
            options.presets.push(require.resolve('@babel/preset-typescript'));
        }
        options.presets.push(require.resolve('@babel/preset-env'));
    }
    return core_1.transform(code, options).code;
}
exports.hook = hook;
function enable(babelrc = false) {
    disable();
    useBabelrc = babelrc;
    revert = pirates_1.addHook(hook, {
        exts: Array.from(extensions_1.PluginExtensions),
        ignoreNodeModules: true
    });
}
exports.enable = enable;
function disable() {
    if (revert) {
        revert();
        revert = null;
    }
}
exports.disable = disable;
//# sourceMappingURL=transpile-requires.js.map